@@ -1,4 +1,4 @@
-#!/usr/bin/env ruby
+#!/usr/local/bin/ruby18
 #
 # svnshell.rb : a Ruby-based shell interface for cruising 'round in
 #               the filesystem.
@@ -125,7 +125,7 @@ class SvnShell
         puts("Invalid argument for #{cmd}: #{args.join(' ')}")
       end
     else
-      puts("Unknown command: #{cmd}")
+      puts("Unknown subcommand: #{cmd}")
       puts("Try one of these commands: ", WORDS.sort.join(" "))
     end
   end
